@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1999-2022 Marco d'Itri <md@linux.it>.
+ * Copyright (C) Marco d'Itri <md@linux.it>.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +40,14 @@
 #include <idna.h>
 #endif
 
+/* prototypes referenced in data.h */
+static void find_referral_server_6bone(char **, const char *);
+static void find_referral_server_apnic(char **, const char *);
+static void find_referral_server_arin(char **, const char *);
+static void find_referral_server_iana(char **, const char *);
+static void find_referral_server_recursive(char **, const char *);
+static void find_referral_server_verisign(char **, const char *);
+
 /* Application-specific */
 #include "version.h"
 #include "data.h"
@@ -311,7 +319,7 @@ int handle_query(const char *hserver, const char *hport,
 	const char *query, const char *flags)
 {
     char *server = NULL, *port = NULL;
-    char *p, *query_string;
+    char *p, *query_string, *old_server, *new_server;
 
     if (hport) {
 	server = strdup(hserver);
@@ -344,19 +352,19 @@ int handle_query(const char *hserver, const char *hport,
 	case 4:
 	    if (verb)
 		printf(_("Using server %s.\n"), server + 1);
-	    sockfd = openconn(server + 1, NULL);
-	    free(server);
-	    server = query_crsnic(sockfd, query);
-	    if (no_recursion)
+	    old_server = server;
+	    server = query_verisign(server, NULL, query);
+	    free(old_server);
+	    if (no_recursion && server)
 		server[0] = '\0';
 	    break;
 	case 8:
 	    if (verb)
 		printf(_("Using server %s.\n"), server + 1);
-	    sockfd = openconn(server + 1, NULL);
-	    free(server);
-	    server = query_afilias(sockfd, query);
-	    if (no_recursion)
+	    old_server = server;
+	    server = query_server(server, NULL, query);
+	    free(old_server);
+	    if (no_recursion && server)
 		server[0] = '\0';
 	    break;
 	case 0x0A:
@@ -388,9 +396,8 @@ int handle_query(const char *hserver, const char *hport,
 	case 0x0E:
 	    if (verb)
 		printf(_("Using server %s.\n"), "whois.iana.org");
-	    sockfd = openconn("whois.iana.org", NULL);
 	    free(server);
-	    server = query_iana(sockfd, query);
+	    server = query_server("whois.iana.org", NULL, query);
 	    break;
 	default:
 	    break;
@@ -408,15 +415,17 @@ int handle_query(const char *hserver, const char *hport,
 	printf(_("Query string: \"%s\"\n\n"), query_string);
     }
 
-    sockfd = openconn(server, port);
-
-    server = do_query(sockfd, query_string);
+    new_server = query_server(server, port, query_string);
+    free(server);
+    if (port)
+	free(port);
     free(query_string);
 
     /* recursion is fun */
-    if (!no_recursion && server && !strchr(query, ' ')) {
-	printf(_("\n\nFound a referral to %s.\n\n"), server);
-	handle_query(server, NULL, query, flags);
+    if (!no_recursion && new_server && !strchr(query, ' ')) {
+	printf(_("\n\nFound a referral to %s.\n\n"), new_server);
+	handle_query(new_server, NULL, query, flags);
+	free(new_server);
     }
 
     return 0;
@@ -494,12 +503,52 @@ const char *match_config_file(const char *s)
 }
 #endif
 
+/*
+ * Check if the argument is an autonomous system number in the form "ASnnnn"
+ * or "AS nnnn".
+ */
+int is_asn(const char *s, int allow_space, const char *suffix_separators)
+{
+    const char *p;
+    int max_numbers = 6;
+    int seen_number = 0;
+
+    if (!strncaseeq(s, "as", 2))
+	return 0;	 /* does not start with "as" (case insensitive) */
+
+    /* skip a single optional space */
+    p = s + 2;
+    if (allow_space && *p == ' ')
+	p++;
+
+    /* all numbers, but not too many */
+    while (*p) {
+	if (*p < '0' || *p > '9') {	/* not a number */
+	    if (!seen_number)
+		return 0;		/* just "as" with no numbers */
+	    if (suffix_separators && strchr(suffix_separators, *p))
+		return 1;		/* has a trailing separator */
+	    return 0;			/* has trailing garbage */
+	}
+
+	seen_number = 1;
+	if (max_numbers-- == 0)
+	    return 0;			/* too many numbers */
+	p++;
+    }
+
+    if (seen_number)
+	return 1;			/* a real ASN */
+    else
+	return 0;			/* trailing garbage */
+}
+
 /* Parses an user-supplied string and tries to guess the right whois server.
  * Returns a dynamically allocated buffer.
  */
 char *guess_server(const char *s)
 {
-    unsigned long ip, as32;
+    unsigned long ip;
     unsigned int i;
     const char *colon, *tld;
 
@@ -508,12 +557,8 @@ char *guess_server(const char *s)
 	unsigned long v6prefix, v6net;
 
 	/* RPSL hierarchical objects */
-	if (strncaseeq(s, "as", 2)) {
-	    if (isasciidigit(s[2]))
-		return strdup(whereas(atol(s + 2)));
-	    else
-		return strdup("");
-	}
+	if (is_asn(s, 0, ":"))
+	    return strdup(whereas(atol(s + 2)));
 
 	v6prefix = strtol(s, NULL, 16);
 
@@ -548,19 +593,14 @@ char *guess_server(const char *s)
 
     /* no dot and no hyphen means it's a NSI NIC handle or ASN (?) */
     if (!strpbrk(s, ".-")) {
-	if (strncaseeq(s, "as", 2) &&		/* it's an AS */
-		(isasciidigit(s[2]) || s[2] == ' '))
-	    return strdup(whereas(atol(s + 2)));
+	if (is_asn(s, 1, NULL))
+	    return strdup(whereas(atol(s + 2))); /* it's an AS */
 	if (*s == '!')	/* NSI NIC handle */
 	    return strdup("whois.networksolutions.com");
 	else
 	    return strdup("\x05"); /* probably a unknown kind of nic handle */
     }
 
-    /* ASN32? */
-    if (strncaseeq(s, "as", 2) && s[2] && (as32 = asn32_to_long(s + 2)) != 0)
-	return strdup(whereas32(as32));
-
     /* smells like an IP? */
     if ((ip = myinet_aton(s))) {
 	for (i = 0; ip_assign[i].serv; i++)
@@ -603,23 +643,10 @@ char *guess_server(const char *s)
     return strdup("\x05");
 }
 
-const char *whereas32(const unsigned long asn)
-{
-    int i;
-
-    for (i = 0; as32_assign[i].serv; i++)
-	if (asn >= as32_assign[i].first && asn <= as32_assign[i].last)
-	    return as32_assign[i].serv;
-    return "\x06";
-}
-
 const char *whereas(const unsigned long asn)
 {
     int i;
 
-    if (asn > 65535)
-	return whereas32(asn);
-
     for (i = 0; as_assign[i].serv; i++)
 	if (asn >= as_assign[i].first && asn <= as_assign[i].last)
 	    return as_assign[i].serv;
@@ -688,18 +715,20 @@ char *queryformat(const char *server, const char *flags, const char *query)
     else if (strchr(query, ' ') || *flags) { }
     else if (streq(server, "whois.denic.de") && in_domain(query, "de"))
 	strcat(buf, "-T dn" DENIC_PARAM_ACE DENIC_PARAM_CHARSET " ");
-    else if (streq(server, "whois.dk-hostmaster.dk") && in_domain(query, "dk"))
+    else if ((streq(server, "whois.punktum.dk") ||
+		streq(server, "whois.dk-hostmaster.dk"))
+	    && in_domain(query, "dk"))
 	strcat(buf, "--show-handles ");
 
     /* mangle and add the query string */
     if (!isripe && streq(server, "whois.nic.ad.jp") &&
-	    strncaseeq(query, "AS", 2) && isasciidigit(query[2])) {
+	    is_asn(query, 1, NULL)) {
 	strcat(buf, "AS ");
 	strcat(buf, query + 2);
     }
     else if (!isripe && streq(server, "whois.arin.net") &&
 	    !strrchr(query, ' ')) {
-	if (strncaseeq(query, "AS", 2) && isasciidigit(query[2])) {
+	if (is_asn(query, 0, NULL)) {
 	    strcat(buf, "a ");
 	    strcat(buf, query + 2);
 	} else if (myinet_aton(query) || strchr(query, ':')) {
@@ -716,8 +745,12 @@ char *queryformat(const char *server, const char *flags, const char *query)
 
     /* ask for english text */
     if (!isripe && (streq(server, "whois.nic.ad.jp") ||
-	    streq(server, "whois.jprs.jp")) && japanese_locale())
-	strcat(buf, "/e");
+	    streq(server, "whois.jprs.jp")) && japanese_locale()) {
+	/* but not if the user already added the /e flag */
+	char *flag = buf + strlen(buf) - 2;
+	if (!streq(flag, "/e"))
+	    strcat(buf, "/e");
+    }
 
     return buf;
 }
@@ -764,55 +797,237 @@ int hide_line(int *hiding, const char *const line)
 	return 0;
 }
 
+static void find_referral_server_6bone(char **referral_server, const char *buf)
+{
+    char nh[256], np[16], nq[1024];
+
+    if (*referral_server)
+	return;
+
+    /* 6bone-style referral:
+     * % referto: whois -h whois.arin.net -p 43 as 1
+     */
+    if (!strneq(buf, "% referto:", 10))
+	return;
+
+    if (sscanf(buf, REFERTO_FORMAT, nh, np, nq) == 3) {
+	/* XXX we are ignoring the new query string */
+	*referral_server = malloc(strlen(nh) + 1 + strlen(np) + 1);
+	sprintf(*referral_server, "%s:%s", nh, np);
+    }
+}
+
+static void find_referral_server_apnic(char **referral_server, const char *buf)
+{
+    /* Possible states of this FSM:
+     * 0: searching for the descr or mnt-by attribute
+     * 1: searching for the mnt-by attribute (descr found)
+     * 2: searching for the descr attribute (mnt-by found)
+     * 3: searching for the end of the object (an empty line)
+     * 4: done: stop searching
+     *
+     * Since the descr attribute is user-provided, consider it only if it
+     * is in an object actually maintained by APNIC.
+     *
+     * If multiple records are returned then it users the referral in the
+     * first valid object found.
+     */
+    static int state = 0;
+    static char *rir_name = NULL;
+    char *p;
+
+    if (state == 4)
+	return;
+
+    if (state <= 2) {
+	if (strneq(buf, "descr:", 6)) {
+	    p = (char *)buf + 6;		/* skip until the colon */
+	    for (; *p == ' ' || *p == '\t'; p++);	/* skip white space */
+	    if (strneq(p, "Transferred to the ", 19)) {
+		rir_name = strdup(p + 19);
+		if ((p = strpbrk(rir_name, " ")))
+		    *p = '\0';
+		if (state == 0)
+		    state = 1;			/* still needs mnt-by */
+		else
+		    state = 3;
+	    }
+	} else if (strneq(buf, "mnt-by:", 7)) {
+	    p = (char *)buf + 7;		/* skip until the colon */
+	    for (; *p == ' ' || *p == '\t'; p++);	/* skip white space */
+	    if (streq(p, "APNIC-STUB")) {
+		if (state == 0)
+		    state = 2;			/* still needs descr */
+		else
+		    state = 3;
+	    }
+	} else if (streq(buf, "")) {
+	    if (state == 1)
+		free(rir_name);
+	    state = 0;			/* start again with a new object */
+	}
+	return;
+    }
+
+    if (state == 3 && streq(buf, "")) {
+	int i;
+	const char *rir_servers[] = {
+	    "AFRINIC",	"whois.afrinic.net",
+	    "ARIN",	"whois.arin.net",
+	    "LACNIC",	"whois.lacnic.net",
+	    "RIPE",	"whois.ripe.net",
+	    NULL,	NULL,
+	};
+
+	state = 4;
+
+	for (i = 0; rir_servers[i]; i += 2)
+	    if (streq(rir_name, rir_servers[i]))
+		*referral_server = strdup(rir_servers[i + 1]);
+	free(rir_name);
+    }
+}
+
+static void find_referral_server_arin(char **referral_server, const char *buf)
+{
+    char *p;
+
+    /* ARIN referrals:
+     * ReferralServer: rwhois://rwhois.fuse.net:4321/
+     * ReferralServer: whois://whois.ripe.net
+     * ReferralServer: whois.ripe.net
+     */
+    if (!strneq(buf, "ReferralServer:", 15))
+	return;
+
+    /* When whois.arin.net returns multiple NetRange records, they will
+     * appear from the less specific one to the most specific one.
+     * Since the range we care about is the most specific one then we
+     * need to use the last ReferralServer field found.
+     */
+    if (*referral_server) {
+	free(*referral_server);
+	*referral_server = NULL;
+    }
+
+    if ((p = strstr(buf, "rwhois://")))
+	*referral_server = strdup(p + 9);
+    else if ((p = strstr(buf, "whois://")))
+	*referral_server = strdup(p + 8);
+    else
+	*referral_server = strdup(buf + 17);
+    if (*referral_server && (p = strpbrk(*referral_server, "/")))
+	*p = '\0';
+}
+
+static void find_referral_server_iana(char **referral_server, const char *buf)
+{
+    const char *p;
+
+    if (*referral_server)
+	return;
+
+    /* IANA referrals:
+     * refer:        whois.apnic.net
+     */
+    if (!strneq(buf, "refer:", 6))
+	return;
+
+    p = buf + 6;				/* skip until the colon */
+    for (; *p == ' ' || *p == '\t'; p++);	/* skip white space */
+    *referral_server = strdup(p);
+}
+
+static void find_referral_server_recursive(char **referral_server, const char *buf)
+{
+    static int state = 0;
+    const char *p;
+
+    if (*referral_server)
+	return;
+
+    if (state == 0 && strneq(buf, "Domain Name:", 12))
+	state = 1;
+    else if (state == 1 && strneq(buf, "Domain Name:", 12))
+	state = 2;
+    else if (state == 1 && strneq(buf, "Registrar WHOIS Server:", 23)) {
+	p = buf + 23;				/* skip until the colon */
+	for (; *p && *p == ' '; p++);		/* skip the spaces */
+	*referral_server = strdup(p);
+	state = 2;
+    }
+}
+
+static void find_referral_server_verisign(char **referral_server, const char *buf)
+{
+    static int state = 0;
+    const char *p;
+
+    if (*referral_server)
+	return;
+
+    /* If there are multiple matches only the server of the first record
+     * is queried */
+    if (state == 0 && strneq(buf, "   Domain Name:", 15)) {
+	state = 1;
+    } else if (state == 0 && strneq(buf, "   Server Name:", 15)) {
+	*referral_server = strdup("");
+	state = 2;
+    } else if (state == 1 && strneq(buf, "   Registrar WHOIS Server:", 26)) {
+	p = buf + 26;				/* skip until the colon */
+	for (; *p && *p == ' '; p++);		/* skip the spaces */
+	*referral_server = strdup(p);
+	state = 2;
+    }
+}
+
 /* returns a string which should be freed by the caller, or NULL */
-char *do_query(const int sock, const char *query)
+char *query_server(const char *server, const char *port, const char *query)
 {
     char *temp, *p, buf[2000];
     FILE *fi;
     int hide = hide_discl;
+    int sock, i;
     char *referral_server = NULL;
+    void (*referral_handler)(char **referral_server, const char *buf) = NULL;
 
     temp = malloc(strlen(query) + 2 + 1);
     strcpy(temp, query);
     strcat(temp, "\r\n");
 
+    for (i = 0; server_referral_handlers[i].name; i++) {
+	/* If the first character is not printable then it represents
+	 * a general category of servers in the list.
+	 */
+	if (server[0] < ' ' &&
+		   strneq(server_referral_handlers[i].name, server, 1)) {
+	    server++;
+	    referral_handler = server_referral_handlers[i].handler;
+	    break;
+	} else if (streq(server_referral_handlers[i].name, server)) {
+	    referral_handler = server_referral_handlers[i].handler;
+	    break;
+	}
+    }
+
+    sock = openconn(server, port);
     fi = fdopen(sock, "r");
+    if (!fi)
+	err_sys("fdopen");
     if (write(sock, temp, strlen(temp)) < 0)
 	err_sys("write");
     free(temp);
 
     while (fgets(buf, sizeof(buf), fi)) {
-	/* 6bone-style referral:
-	 * % referto: whois -h whois.arin.net -p 43 as 1
-	 */
-	if (!referral_server && strneq(buf, "% referto:", 10)) {
-	    char nh[256], np[16], nq[1024];
+	if ((p = strpbrk(buf, "\r\n")))		/* remove the trailing CR/LF */
+	    *p = '\0';
 
-	    if (sscanf(buf, REFERTO_FORMAT, nh, np, nq) == 3) {
-		/* XXX we are ignoring the new query string */
-		referral_server = malloc(strlen(nh) + 1 + strlen(np) + 1);
-		sprintf(referral_server, "%s:%s", nh, np);
-	    }
-	}
-
-	/* ARIN referrals:
-	 * ReferralServer: rwhois://rwhois.fuse.net:4321/
-	 * ReferralServer: whois://whois.ripe.net
-	 */
-	if (!referral_server && strneq(buf, "ReferralServer:", 15)) {
-	    if ((p = strstr(buf, "rwhois://")))
-		referral_server = strdup(p + 9);
-	    else if ((p = strstr(buf, "whois://")))
-		referral_server = strdup(p + 8);
-	    if (referral_server && (p = strpbrk(referral_server, "/\r\n")))
-		*p = '\0';
-	}
+	if (referral_handler)
+	    referral_handler(&referral_server, buf);
 
 	if (hide_line(&hide, buf))
 	    continue;
 
-	if ((p = strpbrk(buf, "\r\n")))
-	    *p = '\0';
 	recode_fputs(buf, stdout);
 	fputc('\n', stdout);
     }
@@ -825,16 +1040,20 @@ char *do_query(const int sock, const char *query)
 	err_quit(_("Catastrophic error: disclaimer text has been changed.\n"
 		   "Please upgrade this program.\n"));
 
+    /* defensive programming: make sure to avoid trivial referral loops */
+    if (referral_server && streq(server, referral_server)) {
+	free(referral_server);
+	referral_server = NULL;
+    }
+
     return referral_server;
 }
 
-char *query_crsnic(const int sock, const char *query)
+char *query_verisign(const char *server, const char *port, const char *query)
 {
-    char *temp, *p, buf[2000];
-    FILE *fi;
-    int hide = hide_discl;
+    char *temp, *p;
+    int sock;
     char *referral_server = NULL;
-    int state = 0;
     int dotscount = 0;
 
     temp = malloc(strlen("domain ") + strlen(query) + 2 + 1);
@@ -848,140 +1067,9 @@ char *query_crsnic(const int sock, const char *query)
     if (dotscount == 1 && !strpbrk(query, "=~ "))
 	strcpy(temp, "domain ");
     strcat(temp, query);
-    strcat(temp, "\r\n");
 
-    fi = fdopen(sock, "r");
-    if (write(sock, temp, strlen(temp)) < 0)
-	err_sys("write");
+    referral_server = query_server(server, port, temp);
     free(temp);
-
-    while (fgets(buf, sizeof(buf), fi)) {
-	/* If there are multiple matches only the server of the first record
-	   is queried */
-	if (state == 0 && strneq(buf, "   Domain Name:", 15))
-	    state = 1;
-	if (state == 0 && strneq(buf, "   Server Name:", 15)) {
-	    referral_server = strdup("");
-	    state = 2;
-	}
-	if (state == 1 && strneq(buf, "   Registrar WHOIS Server:", 26)) {
-	    for (p = buf; *p != ':'; p++);	/* skip until the colon */
-	    for (p++; *p == ' '; p++);		/* skip the spaces */
-	    referral_server = strdup(p);
-	    if ((p = strpbrk(referral_server, "\r\n ")))
-		*p = '\0';
-	    state = 2;
-	}
-
-	/* the output must not be hidden or no data will be shown for
-	   host records and not-existing domains */
-	if (hide_line(&hide, buf))
-	    continue;
-
-	if ((p = strpbrk(buf, "\r\n")))
-	    *p = '\0';
-	recode_fputs(buf, stdout);
-	fputc('\n', stdout);
-    }
-
-    if (ferror(fi))
-	err_sys("fgets");
-    fclose(fi);
-
-    return referral_server;
-}
-
-char *query_afilias(const int sock, const char *query)
-{
-    char *temp, *p, buf[2000];
-    FILE *fi;
-    int hide = hide_discl;
-    char *referral_server = NULL;
-    int state = 0;
-
-    temp = malloc(strlen(query) + 2 + 1);
-    strcpy(temp, query);
-    strcat(temp, "\r\n");
-
-    fi = fdopen(sock, "r");
-    if (write(sock, temp, strlen(temp)) < 0)
-	err_sys("write");
-    free(temp);
-
-    while (fgets(buf, sizeof(buf), fi)) {
-	/* If multiple attributes are returned then use the first result.
-	   This is not supposed to happen. */
-	if (state == 0 && strneq(buf, "Domain Name:", 12))
-	    state = 1;
-	if (state == 1 && strneq(buf, "Registrar WHOIS Server:", 23)) {
-	    for (p = buf; *p != ':'; p++);	/* skip until colon */
-	    for (p++; *p == ' '; p++);		/* skip colon and spaces */
-	    referral_server = strdup(p);
-	    if ((p = strpbrk(referral_server, "\r\n ")))
-		*p = '\0';
-	    state = 2;
-	}
-
-	/* the output must not be hidden or no data will be shown for
-	   host records and not-existing domains */
-	if (hide_line(&hide, buf))
-	    continue;
-
-	if ((p = strpbrk(buf, "\r\n")))
-	    *p = '\0';
-	recode_fputs(buf, stdout);
-	fputc('\n', stdout);
-    }
-
-    if (ferror(fi))
-	err_sys("fgets");
-    fclose(fi);
-
-    if (hide > HIDE_NOT_STARTED && hide != HIDE_TO_THE_END)
-	err_quit(_("Catastrophic error: disclaimer text has been changed.\n"
-		   "Please upgrade this program.\n"));
-
-    return referral_server;
-}
-
-char *query_iana(const int sock, const char *query)
-{
-    char *temp, *p, buf[2000];
-    FILE *fi;
-    char *referral_server = NULL;
-    int state = 0;
-
-    temp = malloc(strlen(query) + 2 + 1);
-    strcpy(temp, query);
-    strcat(temp, "\r\n");
-
-    fi = fdopen(sock, "r");
-    if (write(sock, temp, strlen(temp)) < 0)
-	err_sys("write");
-    free(temp);
-
-    while (fgets(buf, sizeof(buf), fi)) {
-	/* If multiple attributes are returned then use the first result.
-	   This is not supposed to happen. */
-	if (state == 0 && strneq(buf, "refer:", 6)) {
-	    for (p = buf; *p != ':'; p++);	/* skip until colon */
-	    for (p++; *p == ' '; p++);		/* skip colon and spaces */
-	    referral_server = strdup(p);
-	    if ((p = strpbrk(referral_server, "\r\n ")))
-		*p = '\0';
-	    state = 2;
-	}
-
-	if ((p = strpbrk(buf, "\r\n")))
-	    *p = '\0';
-	recode_fputs(buf, stdout);
-	fputc('\n', stdout);
-    }
-
-    if (ferror(fi))
-	err_sys("fgets");
-    fclose(fi);
-
     return referral_server;
 }
 
@@ -1121,7 +1209,7 @@ int connect_with_timeout(int fd, const struct sockaddr *addr,
     return 0;
 }
 
-void NORETURN alarm_handler(int signum)
+void NORETURN alarm_handler(UNUSED int signum)
 {
     close(sockfd);
     err_quit(_("Timeout."));
@@ -1233,6 +1321,12 @@ char *normalize_domain(const char *dom)
     }
 
 #if defined HAVE_LIBIDN || defined HAVE_LIBIDN2
+    /* A leading ! means that this probably is a RADB query, so skip IDN
+     * processing because it would lower-case the command characters.
+     */
+    if (*ret == '!')
+	return ret;
+
     /* find the start of the last word if there are spaces in the query */
     for (p = ret; *p; p++)
 	if (*p == ' ')
@@ -1474,26 +1568,12 @@ unsigned long myinet_aton(const char *s)
     return (a << 24) + (b << 16) + (c << 8) + d;
 }
 
-unsigned long asn32_to_long(const char *s)
-{
-    unsigned long a, b;
-    char junk;
-
-    if (!s)
-	return 0;
-    if (sscanf(s, "%lu.%lu%c", &a, &b, &junk) != 2)
-	return 0;
-    if (a > 65535 || b > 65535)
-	return 0;
-    return (a << 16) + b;
-}
-
 int isasciidigit(const char c)
 {
     return (c >= '0' && c <= '9') ? 1 : 0;
 }
 
-/* http://www.ripe.net/ripe/docs/databaseref-manual.html */
+/* https://www.ripe.net/ripe/docs/databaseref-manual.html */
 
 void NORETURN usage(int error)
 {
